==> dycw_utilities-0.11.7.tar.gz: dycw_utilities-0.11.7/utilities/xarray/__init__.py <==
from collections.abc import Hashable, Mapping
from typing import Any, Optional, Union, cast

from beartype import beartype

from utilities._numbagg import move_exp_nanmean, move_exp_nansum
from utilities.xarray.typing import DataArrayF, DataArrayI


@beartype
def ewma(
    array: Union[DataArrayI, DataArrayF],
    halflife: Optional[Mapping[Hashable, int]] = None,
    /,
    *,
    keep_attrs: Optional[bool] = None,
    **halflife_kwargs: int,
) -> DataArrayF:
    """Compute the EWMA of an array."""
    rolling_exp = array.rolling_exp(halflife, window_type="halflife", **halflife_kwargs)
    return array.reduce(
        _move_exp_nanmean,
        dim=rolling_exp.dim,
        alpha=rolling_exp.alpha,
        keep_attrs=keep_attrs,
    )


@beartype
def _move_exp_nanmean(array: Any, /, *, axis: Any, alpha: Any) -> Any:
    if axis == ():  # pragma: no cover
        return array.astype(float)
    return cast(Any, move_exp_nanmean)(array, axis=axis, alpha=alpha)


@beartype
def exp_moving_sum(
    array: Union[DataArrayI, DataArrayF],
    halflife: Optional[Mapping[Hashable, int]] = None,
    /,
    *,
    keep_attrs: Optional[bool] = None,
    **halflife_kwargs: int,
) -> DataArrayF:
    """Compute the exponentially-weighted moving sum of an array."""
    rolling_exp = array.rolling_exp(halflife, window_type="halflife", **halflife_kwargs)
    return array.reduce(
        _move_exp_nansum,
        dim=rolling_exp.dim,
        alpha=rolling_exp.alpha,
        keep_attrs=keep_attrs,
    )


@beartype
def _move_exp_nansum(array: Any, /, *, axis: Any, alpha: Any) -> Any:
    if axis == ():  # pragma: no cover
        return array.astype(float)
    return cast(Any, move_exp_nansum)(array, axis=axis, alpha=alpha)

==> dycw_utilities-0.12.2.tar.gz: dycw_utilities-0.12.2/utilities/xarray/__init__.py <==
from __future__ import annotations

from collections.abc import Hashable, Mapping
from typing import Any, cast

from utilities._numbagg import move_exp_nanmean, move_exp_nansum
from utilities.xarray.typing import DataArrayF, DataArrayI


def ewma(
    array: DataArrayI | DataArrayF,
    halflife: Mapping[Hashable, int] | None = None,
    /,
    *,
    keep_attrs: bool | None = None,
    **halflife_kwargs: int,
) -> DataArrayF:
    """Compute the EWMA of an array."""
    rolling_exp = array.rolling_exp(
        halflife, window_type="halflife", **halflife_kwargs
    )
    return array.reduce(
        _move_exp_nanmean,
        dim=rolling_exp.dim,
        alpha=rolling_exp.alpha,
        keep_attrs=keep_attrs,
    )


def _move_exp_nanmean(array: Any, /, *, axis: Any, alpha: Any) -> Any:
    if axis == ():  # pragma: no cover
        return array.astype(float)
    return cast(Any, move_exp_nanmean)(array, axis=axis, alpha=alpha)


def exp_moving_sum(
    array: DataArrayI | DataArrayF,
    halflife: Mapping[Hashable, int] | None = None,
    /,
    *,
    keep_attrs: bool | None = None,
    **halflife_kwargs: int,
) -> DataArrayF:
    """Compute the exponentially-weighted moving sum of an array."""
    rolling_exp = array.rolling_exp(
        halflife, window_type="halflife", **halflife_kwargs
    )
    return array.reduce(
        _move_exp_nansum,
        dim=rolling_exp.dim,
        alpha=rolling_exp.alpha,
        keep_attrs=keep_attrs,
    )


def _move_exp_nansum(array: Any, /, *, axis: Any, alpha: Any) -> Any:
    if axis == ():  # pragma: no cover
        return array.astype(float)
    return cast(Any, move_exp_nansum)(array, axis=axis, alpha=alpha)
